--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 12.10.2020

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


local loadError
local versionString = "0.0.0.0"

local modName = g_currentModName
local modDirectory = g_currentModDirectory

local function finalizeVehicleTypes(vehicleTypeManager)
    local specializationName = string.format("%s.rideableFirstPersonCamera", modName)
    local specializationObject = g_specializationManager:getSpecializationObjectByName(specializationName)

    if specializationObject ~= nil then
        for typeName, typeEntry in pairs(vehicleTypeManager.vehicleTypes) do
            if specializationObject.prerequisitesPresent(typeEntry.specializations) then
                g_vehicleTypeManager:addSpecialization(typeName, specializationName)
            end
        end
    end
end

local function init()
    local mod = g_modManager:getModByName(modName)
    versionString = tostring(mod.version) or versionString

    if mod.modName == "FS19_HorseRidingCamera" or mod.modName == "FS19_HorseRidingCamera_update" and mod.author:len() == 3 then
        g_specializationManager:addSpecialization("rideableFirstPersonCamera", "RideableFirstPersonCamera", modDirectory .. "scripts/specializations/RideableFirstPersonCamera.lua")
        VehicleTypeManager.finalizeVehicleTypes = Utils.prependedFunction(VehicleTypeManager.finalizeVehicleTypes, finalizeVehicleTypes)
    else
        loadError = {
            startUpdateTime = 2000,

            update = function(self, dt)
                self.startUpdateTime = self.startUpdateTime - dt

                if self.startUpdateTime < 0 then
                    local text = string.format(g_i18n:getText("HRC_loadError", modName), modName, mod.author)

                    if g_dedicatedServerInfo == nil then
                        if not g_gui:getIsGuiVisible() then
                            g_gui:showYesNoDialog({
                                title = string.format("%s - Version %s", mod.title, versionString),
                                text = text,
                                dialogType = DialogElement.TYPE_LOADING,
                                callback = self.openModHubLink,
                                target = nil,
                                yesText = g_i18n:getText("button_ok"),
                                noText = g_i18n:getText("button_modHubDownload")
                            })
                        end
                    else
                        print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0" .. "\n")
                        self.openModHubLink(true)
                    end
                end
            end,

            openModHubLink = function(ignore)
                if ignore == false then
                    local language = g_languageShort
                    local link = "mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                    if language == "de" or language == "fr" then
                        link = "mods.php?lang=" .. language .. "&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                    end

                    openWebFile(link, "")
                end

                removeModEventListener(loadError)
                loadError = nil
            end
        }

        addModEventListener(loadError)
    end
end

init()
